@@ -7,7 +7,6 @@ import socket
 import ssl
 from datetime import timedelta
 from typing import Callable, Dict, List, Optional, Tuple, Union, Coroutine, Any
-
 from .color_value import ColorMode, WarmDimmingColorValue
 
 
@@ -47,7 +46,9 @@ class Smartbridge:
     """
 
     def __init__(
-        self, connect: Callable[[], Coroutine[Any, Any, LeapProtocol]]
+        self,
+        connect: Callable[[], Coroutine[Any, Any, LeapProtocol]],
+        on_connect_callback: Optional[Callable[[], None]] = None,
     ) -> None:
         """Initialize the Smart Bridge."""
         self.devices: Dict[str, dict] = {}
@@ -69,6 +70,7 @@ class Smartbridge:
         self._leap: Optional[LeapProtocol] = None
         self._monitor_task: Optional[asyncio.Task] = None
         self._ping_task: Optional[asyncio.Task] = None
+        self._on_connect_callback = on_connect_callback
 
     @property
     def logged_in(self):
@@ -83,7 +85,7 @@ class Smartbridge:
             and self._login_completed.exception() is None
         )
 
-    async def connect(self):
+    async def connect(self) -> None:
         """Connect to the bridge."""
         # reset any existing connection state
         if self._login_task is not None:
@@ -135,6 +137,7 @@ class Smartbridge:
         certfile: str,
         ca_certs: str,
         port: int = LEAP_PORT,
+        on_connect_callback: Optional[Callable[[], None]] = None,
     ) -> "Smartbridge":
         """Initialize the Smart Bridge using TLS over IPv4."""
 
@@ -151,7 +154,7 @@ class Smartbridge:
             )
             return res
 
-        return cls(_connect)
+        return cls(_connect, on_connect_callback)
 
     def add_subscriber(self, device_id: str, callback_: Callable[[], None]):
         """
@@ -625,6 +628,9 @@ class Smartbridge:
             self._leap.subscribe_unsolicited(self._handle_unsolicited)
             _LOG.debug("Successfully connected to Smart Bridge.")
 
+            if self._on_connect_callback:
+                self._on_connect_callback()
+
             if self._login_task is not None:
                 self._login_task.cancel()
 
@@ -1436,10 +1442,9 @@ class Smartbridge:
     async def close(self):
         """Disconnect from the bridge."""
         _LOG.info("Processing Smartbridge.close() call")
-        if self._monitor_task is not None and not self._monitor_task.cancelled():
-            self._monitor_task.cancel()
-        if self._ping_task is not None and not self._ping_task.cancelled():
-            self._ping_task.cancel()
+        for task in (self._monitor_task, self._ping_task, self._login_task):
+            if task is not None and not task.done():
+                task.cancel()
 
 
 def _format_duration(duration: timedelta) -> str:
